//IMPORTS
Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

String.prototype.startsWith = function(str) {
    return (this.match("^"+str)==str)
}

const TB_VERSION = 5;
const TB_BROWSER = "ff";

const DO_DEBUG = false;
function debug(s) {
    if (DO_DEBUG) {
        dump( new Date() + ": BktbConnect: " + s +"\n");
    }
}


/*********************************************************** 
constants 
 ***********************************************************/
  
// reference to the interface defined in nsIBktbConnect.idl
const nsIBktbConnect = Components.interfaces.nsIBktbConnect;
  
// reference to the required base interface that all components must support  
const nsISupports = Components.interfaces.nsISupports;  
  
// UUID uniquely identifying our component  
// You can get from: http://kruithof.xs4all.nl/uuid/uuidgen here  
const CLASS_ID = Components.ID("{a2374fb0-4c76-11df-9879-0800200c9a66}");
  
// description  
const CLASS_NAME = "BktbConnect Javascript XPCOM Component";
  
// textual unique identifier  
const CONTRACT_ID = "@bundeskampf.com/bktbconnect;1";


//const BASE_URL = "http://bundeskampf.dev.spielwerk.eu/";
const BASE_URL = "http://www.bundeskampf.com/";

// URLs that force a data update
var forceUpdateURLs = new Array();
forceUpdateURLs.push(BASE_URL + "quests/start");
forceUpdateURLs.push(BASE_URL + "quests/doQuest");
forceUpdateURLs.push(BASE_URL + "fights/start");
forceUpdateURLs.push(BASE_URL + "fights/results");
forceUpdateURLs.push(BASE_URL + "signups/logout");
forceUpdateURLs.push(BASE_URL + "toolbars/trigger");
forceUpdateURLs.push(BASE_URL + "arbeitsamt/cancel");
forceUpdateURLs.push(BASE_URL + "arbeitsamt/finish");
forceUpdateURLs.push(BASE_URL + "arbeitsamt/serve");

var UPDATE_URL          = BASE_URL + 'toolbars/update/' + TB_VERSION + "/" + TB_BROWSER;
var obj;    // the ajax obj
var req;    // the RemoteRequest Object
var lastUpdate = 0;
var updateEvery = 10000;
var toolbars = new Array();

var now = function() {
    return parseInt(new Date().getTime());
}

 
/*********************************************************** 
class definition 
 ***********************************************************/
  
//class constructor  
function BktbConnect() {
    // If you only need to access your component from Javascript, uncomment the following line:
    this.wrappedJSObject = this;
}  
  
// class definition  
BktbConnect.prototype = {
  classID: Components.ID("a2374fb0-4c76-11df-9879-0800200c9a66"),
  classDescription: "Bundeskampftoolbar 1.1",
  contractID: "@bundeskampf.com/bktbconnect;1",

  QueryInterface: XPCOMUtils.generateQI([]),

    isRegistered: false,

    // define the function we want to expose in our interface
    hello: function() {
        return "Hello World AAAAA!";
    },

    getUsr: function () {
        return "This is user 1";
    },
  
    QueryInterface: function(aIID)
    {
        if (!aIID.equals(nsIBktbConnect) &&
            !aIID.equals(nsISupports))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    },

    /* Toolbar Interaction */
    getUpdate: function() {
        this.updateUser();

        //return usr;
    },



    /* AJAX Update */
    updateUser: function() {
        debug("updateUser diff=" + (now() - lastUpdate) + " lastU=" + lastUpdate + " updateEvery=" + updateEvery);
        if (
        ((now() - lastUpdate) > updateEvery)
    ) {
            this.requestUpdate()
        } else {
            // fixed new window no update bug
            bktbConnect.notifyToolbars();
        }
    },

    requestUpdate: function() {
        lastUpdate = now();
        
        debug("requestUpdate");
        req = Components.classes["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Components.interfaces.nsIXMLHttpRequest);

        req.open("GET", UPDATE_URL, true);
        req.send(null);

        req.onload = this.updateUserOnLoad;
        req.onerror = this.updateUserOnError;
    },

    updateUserOnLoad: function(aEvent) {
        var text = aEvent.target.responseText;
        obj = JSON.parse(text);

		obj.timeCorrection = (obj.serverTime - parseInt(new Date().getTime()/1000));

        if (obj.updateEvery) {
            //debug('resetting interval to: ' + obj.updateEvery);
            updateEvery = obj.updateEvery;
        }

        bktbConnect.notifyToolbars();

        req = null;
    },

    updateUserOnError: function(aEvent) {
        //debug("updateUserOnError");
        obj = false;

        bktbConnect.notifyToolbars();

        req = null;
    },



    /* Toolbar Observers */
    addToolbar: function(toolbar) {
        toolbars.push(toolbar);
        if (obj) {
            //toolbar(obj);
        }
    },

    notifyToolbars: function() {
        var r = false;

        //debug("notify numToolbars: " + toolbars.length);
        for (var i = 0; i < toolbars.length; i++) {
            r = toolbars[i](obj);
            if (!r) {
                this.removeToolbar(i);
            }
        }
    },

    removeToolbar: function(index) {
        toolbars.splice(index, 1);
    },



    /* HTTP Observer */
    observe: function(subject, topic, data) {
        if (typeof Components == 'undefined')
            return;

        if (topic == "http-on-examine-response") {
            subject.QueryInterface(Components.interfaces.nsIHttpChannel);
            var url = subject.URI.spec;

            for (var i=0; i < forceUpdateURLs.length; i++) {
                if (url.startsWith(forceUpdateURLs[i])) {
                    debug("force update on: " + url);
                    this.requestUpdate();
                    //this.notifyToolbars();
                    //bktb.updateUser();
                }
            }
        }
    },

    get observerService() {
        return Components.classes["@mozilla.org/observer-service;1"]
        .getService(Components.interfaces.nsIObserverService);
    },

    register: function() {
        if (!this.isRegistered) {
            this.isRegistered = true;
            this.observerService.addObserver(this, "http-on-examine-response", false);
        }
    },

    unregister: function() {
        this.observerService.removeObserver(this, "http-on-examine-response");
    }
};  
  
/*********************************************************** 
class factory 
 
This object is a member of the global-scope Components.classes. 
It is keyed off of the contract ID. Eg: 
 
myBktbConnect = Components.classes["@dietrich.ganx4.com/BktbConnect;1"].
                          createInstance(Components.interfaces.nsIBktbConnect);
 
 ***********************************************************/
var BktbConnectFactory = {
    createInstance: function (aOuter, aIID)
    {
        if (aOuter != null)
            throw Components.results.NS_ERROR_NO_AGGREGATION;
        return (new BktbConnect()).QueryInterface(aIID);
    }
};  
  
/*********************************************************** 
module definition (xpcom registration) 
 ***********************************************************/
var BktbConnectModule = {
    registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
    {
        aCompMgr = aCompMgr.
            QueryInterface(Components.interfaces.nsIComponentRegistrar);
        aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME,
        CONTRACT_ID, aFileSpec, aLocation, aType);
    },
  
    unregisterSelf: function(aCompMgr, aLocation, aType)
    {
        aCompMgr = aCompMgr.
            QueryInterface(Components.interfaces.nsIComponentRegistrar);
        aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
    },
    
    getClassObject: function(aCompMgr, aCID, aIID)
    {
        if (!aIID.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
  
        if (aCID.equals(CLASS_ID))
            return BktbConnectFactory;
  
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },
  
    canUnload: function(aCompMgr) {
        return true;
    }
};  
  
/*********************************************************** 
module initialization 
 
When the application registers the component, this function 
is called. 
 ***********************************************************/

if ("generateNSGetFactory" in XPCOMUtils)
  var NSGetFactory = XPCOMUtils.generateNSGetFactory([BktbConnect]);  // 2.0+
else
  var NSGetModule = XPCOMUtils.generateNSGetModule([BktbConnect]);    // 1.9.x


var bktbConnect = new BktbConnect();
/*
function updateObj(newObj) {
    obj = newObj;
    debug(obj);
    .notifyToolbars();
}
*/